@@ -106,6 +106,30 @@ sub getline_FileTail
 
 =cut #-------------------------- File::Tail -----------------------------END-
 
+=cut #----------------------- Read log on stdin -----------------------START-
+# If you want to have syslog run pop-before-smtp and send it log lines via
+# stdin, comment-out the two surrounding =cut lines and setup your syslog
+# to have an entry like this:
+#
+#    mail.info                     | exec /usr/local/sbin/pop-before-smtp
+#
+# Also, make sure there's not a pop-before-smtp init script installed.
+
+$file_tail{'name'} = '-';   # Just for the sake of --dumpconfig
+
+$tail_init_func = sub { };
+$tail_getline_func = \&getline_stdin;
+
+sub getline_stdin
+{
+    my $line = <STDIN>;
+    if (!defined $line) {
+        die "EOF on stdin -- exiting\n";
+    }
+    $line;
+}
+=cut #----------------------- Read log on stdin -------------------------END-
+
 ############################# START OF PATTERNS #############################
 #
 # Pick one of these values for the $pat variable OR define a subroutine
@@ -175,13 +199,13 @@ sub getline_FileTail
 
 # For Cyrus (including a tweak for IP addrs that don't resolve):
 #$pat = '^[LOGTIME] \S+ (?:cyrus/)?(?:pop3|imap)[ds]?\[\d+\]: ' .
-#    'login: .*?\[[:f]*(\d+\.\d+\.\d+\.\d+)\]';
+#    'login: .*?\[[:f]*(\d+\.\d+\.\d+\.\d+)\] .* User logged in';
 
 # For Courier-POP3 and Courier-IMAP:
 #$pat = '^[LOGTIME] (?:\[|\S+ )(?:pop3|imap|couriertcp)(?:d|d-ssl|login)\]?: ' .
 #    'LOGIN, user=\S+, ip=\[[:f]*(\d+\.\d+\.\d+\.\d+)\]';
 #$out_pat = '^[LOGTIME] (?:\[|\S+ )(?:pop3|imap|couriertcp)(?:d|d-ssl|login)\]?: ' .
-#    '(?:LOGOUT|DISCONNECTED), user=\S+, ip=\[[:f]*(\d+\.\d+\.\d+\.\d+)\]';
+#    '(?:LOGOUT|TIMEOUT|DISCONNECTED), user=\S+, ip=\[[:f]*(\d+\.\d+\.\d+\.\d+)\]';
 
 # For qmail's pop3d:
 #$pat = '^[LOGTIME] \S+ vpopmail\[\d+\]: vchkpw: ' .
@@ -274,6 +298,7 @@ $login[1] = '... define an extra login pattern here ...';
 
 foreach (@login, @logout) {
     next unless defined $_;
+    s/\[LOGTIME\]/$logtime_pat/g;
     $_ = qr/$_/;	# Pre-compile the regular expressions.
 }
 
@@ -504,7 +529,9 @@ sub sync_sendmail
 {
     $dbh->sync and die "$0: sync $dbfile: $!\n";
 
-    while ($signal_sendmail && !kill(1, $sendmail_pid)) {
+    while ($signal_sendmail) {
+	$log_func->('debug', "about to signal $sendmail_pid") if $debug;
+	last if kill(1, $sendmail_pid);
 	open(PID, $pid_file) || die "Unable to open $pid_file: $!";
 	$_ = <PID>;
 	my($new_pid) = /(\d+)/;
@@ -512,6 +539,7 @@ sub sync_sendmail
 	if ($new_pid == $sendmail_pid) {
 	    die "Unable to signal sendmail to reread the database.\n";
 	}
+	$log_func->('debug', "discovered new sendmail_pid: $new_pid") if $debug;
 	$sendmail_pid = $new_pid;
     }
 }
